package org.jeecg.modules.flowpath.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.exception.JeecgBootException;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.modules.flowpath.convert.FlowPathConvert;
import org.jeecg.modules.flowpath.dto.ItdmPassRunFlowPath;
import org.jeecg.modules.flowpath.dto.ItdmRejectDQRunFlowPath;
import org.jeecg.modules.flowpath.dto.ItdmRejectZDRunFlowPath;
import org.jeecg.modules.flowpath.dto.ItdmSaveRunFlowPath;
import org.jeecg.modules.flowpath.entity.ItdmRunFlowPath;
import org.jeecg.modules.flowpath.entity.ItdmRunFlowPathStep;
import org.jeecg.modules.flowpath.mapper.ItdmRunFlowPathLogMapper;
import org.jeecg.modules.flowpath.mapper.ItdmRunFlowPathMapper;
import org.jeecg.modules.flowpath.mapper.ItdmRunFlowPathStepMapper;
import org.jeecg.modules.flowpath.service.IItdmRunFlowPathStepService;
import org.jeecg.modules.flowpath.vo.ItdmBuohuiRunFlowPathStepVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description: 运行流程步骤表
 * @Author: jeecg-boot
 * @Date: 2023-07-20
 * @Version: V1.0
 */
@Service
public class ItdmRunFlowPathStepServiceImpl extends ServiceImpl<ItdmRunFlowPathStepMapper, ItdmRunFlowPathStep> implements IItdmRunFlowPathStepService {


    @Autowired
    @SuppressWarnings("all")
    private ItdmRunFlowPathMapper itdmRunFlowPathMapper;

    @Autowired
    @SuppressWarnings("all")
    private ItdmRunFlowPathLogMapper logMapper;

    @Autowired
    private ItdmFZRunFlowPathLogService logService;

    @Override
    public void saveData(ItdmSaveRunFlowPath saveRunFlowPath) {
        ItdmRunFlowPathStep select = this.baseMapper.selectById(saveRunFlowPath.getRunFlowPathStep());
        ItdmRunFlowPath itdmRunFlowPath = this.itdmRunFlowPathMapper.selectById(select.getRunFlowPath());
        if (!itdmRunFlowPath.getDqSetp().equals(select.getFlowPathSetp())) {
            throw new JeecgBootException("当前节点不是在进行中");
        }
        ItdmRunFlowPathStep itdmRunFlowPathStep = new ItdmRunFlowPathStep();
        itdmRunFlowPathStep.setId(saveRunFlowPath.getRunFlowPathStep());
        itdmRunFlowPathStep.setSaveData(saveRunFlowPath.getSaveData());

        this.baseMapper.updateById(itdmRunFlowPathStep);

    }

    @Override
    @Transactional
    public void pass(ItdmSaveRunFlowPath saveRunFlowPath) {

        ItdmRunFlowPathStep select = this.baseMapper.selectById(saveRunFlowPath.getRunFlowPathStep());
        select.setSaveData(saveRunFlowPath.getSaveData());
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        select.setFqUser(user.getUsername());
        this.baseMapper.updateById(select);
        ItdmRunFlowPathStep nextstep = this.baseMapper.selectOne(Wrappers.lambdaQuery(ItdmRunFlowPathStep.class)
                .eq(ItdmRunFlowPathStep::getRunFlowPath, select.getRunFlowPath())
                .eq(ItdmRunFlowPathStep::getSort, Integer.parseInt(select.getSort()) + 1)

        );
        ItdmRunFlowPath itdmRunFlowPath = this.itdmRunFlowPathMapper.selectById(select.getRunFlowPath());
        if (!itdmRunFlowPath.getDqSetp().equals(select.getFlowPathSetp())) {
            throw new JeecgBootException("当前节点不是在进行中");
        }

        itdmRunFlowPath.setId(select.getRunFlowPath());
        itdmRunFlowPath.setDqSetp(nextstep.getFlowPathSetp());
        itdmRunFlowPath.setDqSetpUserIds(nextstep.getUserIds());
        itdmRunFlowPath.setDqSetpRoleId(nextstep.getRoleId());
        itdmRunFlowPathMapper.updateById(itdmRunFlowPath);

        logService.insert(select.getRunFlowPath(), select.getId(), nextstep.getId(), "通过", "");

    }

    @Override
    public void pass1(ItdmPassRunFlowPath itdmRunFlowPathStep) {

        ItdmRunFlowPath itdmRunFlowPath = this.itdmRunFlowPathMapper.selectById(itdmRunFlowPathStep.getRunFlowPath());
        ItdmRunFlowPathStep select = this.baseMapper.selectOne(Wrappers.lambdaQuery(ItdmRunFlowPathStep.class)
                .eq(ItdmRunFlowPathStep::getRunFlowPath, itdmRunFlowPath.getId())
                .eq(ItdmRunFlowPathStep::getFlowPathSetp, itdmRunFlowPath.getDqSetp())

        );
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        select.setFqUser(user.getUsername());
        this.baseMapper.updateById(select);
        ItdmRunFlowPathStep nextstep = this.baseMapper.selectOne(Wrappers.lambdaQuery(ItdmRunFlowPathStep.class)
                .eq(ItdmRunFlowPathStep::getRunFlowPath, select.getRunFlowPath())
                .eq(ItdmRunFlowPathStep::getSort, Integer.parseInt(select.getSort()) + 1)

        );
        itdmRunFlowPath.setId(select.getRunFlowPath());
        itdmRunFlowPath.setDqSetp(nextstep.getFlowPathSetp());
        itdmRunFlowPath.setDqSetpUserIds(nextstep.getUserIds());
        itdmRunFlowPath.setDqSetpRoleId(nextstep.getRoleId());
        itdmRunFlowPathMapper.updateById(itdmRunFlowPath);

        logService.insert(select.getRunFlowPath(), select.getId(), nextstep.getId(), "通过", "");
    }

    @Override
    public void rejectDQ(ItdmRejectDQRunFlowPath itdmRejectDQRunFlowPath) {
        ItdmRunFlowPath itdmRunFlowPath = this.itdmRunFlowPathMapper.selectById(itdmRejectDQRunFlowPath.getRunFlowPath());
        ItdmRunFlowPathStep select = this.baseMapper.selectOne(Wrappers.lambdaQuery(ItdmRunFlowPathStep.class)
                .eq(ItdmRunFlowPathStep::getRunFlowPath, itdmRunFlowPath.getId())
                .eq(ItdmRunFlowPathStep::getFlowPathSetp, itdmRunFlowPath.getDqSetp())

        );


        ItdmRunFlowPathStep prvestep = this.baseMapper.selectOne(Wrappers.lambdaQuery(ItdmRunFlowPathStep.class)
                .eq(ItdmRunFlowPathStep::getRunFlowPath, select.getRunFlowPath())
                .eq(ItdmRunFlowPathStep::getSort, Integer.parseInt(select.getSort()) - 1)

        );

        itdmRunFlowPath.setDqSetp(prvestep.getFlowPathSetp());
        itdmRunFlowPath.setDqSetpUserIds(prvestep.getUserIds());
        itdmRunFlowPath.setDqSetpRoleId(prvestep.getRoleId());

        itdmRunFlowPathMapper.updateById(itdmRunFlowPath);
        logService.insert(select.getRunFlowPath(), select.getId(), prvestep.getId(), "驳回上一节点", itdmRejectDQRunFlowPath.getYuanyin());

    }


    @Override
    public List<ItdmBuohuiRunFlowPathStepVO> yrunList(String runFlowPath) {
        ItdmRunFlowPath itdmRunFlowPath = this.itdmRunFlowPathMapper.selectById(runFlowPath);
        ItdmRunFlowPathStep select = this.baseMapper.selectOne(Wrappers.lambdaQuery(ItdmRunFlowPathStep.class)
                .eq(ItdmRunFlowPathStep::getRunFlowPath, itdmRunFlowPath.getId())
                .eq(ItdmRunFlowPathStep::getFlowPathSetp, itdmRunFlowPath.getDqSetp())

        );
        List<ItdmRunFlowPathStep> prvestep = this.baseMapper.selectList(Wrappers.lambdaQuery(ItdmRunFlowPathStep.class)
                .eq(ItdmRunFlowPathStep::getRunFlowPath, select.getRunFlowPath())
                .lt(ItdmRunFlowPathStep::getSort, Integer.parseInt(select.getSort()))

        );

        return prvestep.stream().map(FlowPathConvert.INSTANCE::to).collect(Collectors.toList());
    }


    @Override
    public void rejectzd(ItdmRejectZDRunFlowPath zdRunFlowPath) {
        ItdmRunFlowPath itdmRunFlowPath = this.itdmRunFlowPathMapper.selectById(zdRunFlowPath.getRunFlowPath());
        ItdmRunFlowPathStep prvestep = this.baseMapper.selectById(zdRunFlowPath.getRunFlowPathSept());
        itdmRunFlowPath.setDqSetp(prvestep.getFlowPathSetp());
        itdmRunFlowPath.setDqSetpUserIds(prvestep.getUserIds());
        itdmRunFlowPath.setDqSetpRoleId(prvestep.getRoleId());
        itdmRunFlowPathMapper.updateById(itdmRunFlowPath);
        logService.insert(prvestep.getRunFlowPath(), itdmRunFlowPath.getDqSetp(), prvestep.getId(), "驳回指定节点", zdRunFlowPath.getYuanyin());


    }


    @Override
    public void jieshu(String s) {
        ItdmRunFlowPath itdmRunFlowPath = this.itdmRunFlowPathMapper.selectById(entityClass);
        itdmRunFlowPath.setStatus("1");

        itdmRunFlowPathMapper.updateById(itdmRunFlowPath);


    }
}
